/*****************************************************************
 * Copyright (c) ParaSoft Corporation 1991.  All rights reserved.*
 * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF ParaSoft       *
 * The copyright notice above does not evidence any              *
 * actual or intended publication of such source code.           *
 *****************************************************************/

#include "extern.h"

/******************************** C H E C K **************************************/

static int if_expr_must_be_logical (etype)
Etype *etype;
{
	if (etype->type != TYP_LOGICAL) {
		sprintf (serr, "expression in IF statement must be LOGICAL, is %s"
		, etype_name (etype));
		fatal (serr);  /* E_8_3_6 */
	}
}

static int if_expr_must_be_scalar (shape)
Shape *shape;
{
	if (SHAPE_IS_NOT_NULL (shape)) {
		sprintf (serr, "expression in IF statement must be scalar, is of rank %d"
		, shape->rank);
		fatal (serr);  /* E_8_3_6 */
	}
}

Node *check_IfStmt (root)
Node *root;
{
	Typ *typ = root->IfStmt.Expr->com.typ;
	int type;

	if_expr_must_be_logical (typ->etype);
	if_expr_must_be_scalar (typ->shape);

	type = root->IfStmt.Stmt->com.type;

	if (type == IF_STMT
	||  type == END_PROGRAM_STMT
	||  type == END_FUNCTION_STMT
	||  type == END_SUBROUTINE_STMT
	||  !order_is_action_stmt (type)) {
		sprintf (serr, "the action statement in an IF statement cannot be a %s"
		, node_Symbol (type));
		fatal (serr);  /* E_8_3_6-7 */
	}

	return root;
}

Node *check_IfThenStmt (root)
Node *root;
{
	Typ *typ = root->IfThenStmt.Expr->com.typ;
	Node *ifc;
	Node *label;

	if_expr_must_be_logical (typ->etype);
	if_expr_must_be_scalar (typ->shape);

	ifc = make (IF_CONSTRUCT);
	block_push (ifc);
	BLOCK_ADD_STMTS_TO (ifc->IfConstruct.IfBlock);
	ifc->IfConstruct.IfThenStmt = root;

	label = ifc->IfConstruct.label = se_current_statement->Statement.IntegerConstant;
	ifc->IfConstruct.file = input_filename;
	ifc->IfConstruct.line = zzlineno;

	IGNORE_RETURN;
}
	
Node *check_ElseIfStmt(root)
Node *root;
{
	Typ *typ = root->IfThenStmt.Expr->com.typ;
	struct S_IfConstruct *sifc;
	Node *ifc, *else_block, *temp;
	char *if_name, *elseif_name;

	if_expr_must_be_logical (typ->etype);
	if_expr_must_be_scalar (typ->shape);

	if ((ifc = block_match ("ELSE IF", IF_CONSTRUCT)) == NN)
		IGNORE_RETURN;

	sifc = &ifc->IfConstruct;

	if (ifc->IfConstruct.ElseStmt != NN)
		fatal ("misplaced ELSE IF");

	else_block = make (ELSE_IF_STMT_BLOCK);

	BLOCK_ADD_STMTS_TO (else_block->ElseIfStmtBlock.ElseIfBlock);
	else_block->ElseIfStmtBlock.ElseIfStmt = root;

	ifc->IfConstruct.ElseIfStmtBlockList = list_add_beg (
	  ifc->IfConstruct.ElseIfStmtBlockList
	, else_block
	, ELSE_IF_STMT_BLOCK_LIST);

/* checks construct-name stuff */

	if_name = (sifc->IfThenStmt->IfThenStmt.Name1 != NN)
	? sifc->IfThenStmt->IfThenStmt.Name->com.str : CN;
	elseif_name = (root->ElseIfStmt.Name1 != NN) ? root->ElseIfStmt.Name1->com.str : CN;

	if (elseif_name != CN) {
		if (if_name == CN)
			fatal ("this ELSE IF should not have a construct-name");
		else if (U_strcmp_nocase (if_name, elseif_name))
			fatal ("construct-name in ELSE IF doesn't match that of the IF");
	}

	IGNORE_RETURN;
}

Node *check_ElseStmt (root)
Node *root;
{
	Node *ifc;
	struct S_IfConstruct *sifc;
	char *if_name, *else_name;

	if ((ifc = block_match ("ELSE", IF_CONSTRUCT)) == NN)
		IGNORE_RETURN;

	sifc = &ifc->IfConstruct;

	if (ifc->IfConstruct.ElseStmt != NN)
		fatal ("too many ELSEs");

	BLOCK_ADD_STMTS_TO (ifc->IfConstruct.ElseBlock);
	ifc->IfConstruct.ElseStmt = root;

/* checks construct-name stuff */

	if_name = (sifc->IfThenStmt->IfThenStmt.Name1 != NN)
	? sifc->IfThenStmt->IfThenStmt.Name->com.str : CN;
	else_name = (root->ElseStmt.Name != NN) ? root->ElseStmt.Name->com.str : CN;

	if (else_name != CN) {
		if (if_name == CN)
			fatal ("this ELSE should not have a construct-name");
		else if (U_strcmp_nocase (if_name, else_name))
			fatal ("construct-name in ELSE doesn't match that of the IF");
	}

	IGNORE_RETURN;
}

Node *check_EndIfStmt(root)
Node *root;
{
	Node *ifc;
	struct S_IfConstruct *sifc;
	char *if_name, *endif_name;

	if ((ifc = block_match ("END IF", IF_CONSTRUCT)) == NN)
		IGNORE_RETURN;

	ifc = block_pop ();
	sifc = &ifc->IfConstruct;
	sifc->EndIfStmt = root;
	sifc->label_endif = CURRENT_LABEL;

/* checks construct-name stuff */

	if_name = (sifc->IfThenStmt->IfThenStmt.Name1 != NN)
	? sifc->IfThenStmt->IfThenStmt.Name->com.str : CN;
	endif_name = (root->EndIfStmt.Name != NN) ? root->EndIfStmt.Name->com.str : CN;

	if (if_name != CN && endif_name != CN) {
		if (U_strcmp_nocase (if_name, endif_name))
			fatal ("construct-name in END IF doesn't match that of the IF");
	}
	else if (if_name != CN && endif_name == CN) {
		fatal ("missing construct-name in END IF statement");
	}
	else if (if_name == CN && endif_name != CN) {
		fatal ("this END IF should not have a construct-name");
	}
	
	block_add_stmt_with_label (ifc, sifc->label, sifc->file, sifc->line);
	sifc->trans = TRN;

	IGNORE_RETURN;
}

/******************************** T R A N S **************************************/

/******************************** W R I T E **************************************/

static U_DEFINE_STACK (selse, Node*, 5)

static write_elseif_block (elseif_block)
Node *elseif_block;
{
	struct S_ElseIfStmtBlock *eib = &elseif_block->ElseIfStmtBlock;

	if (trans_tree_has_stmts (eib->ElseIfStmt)) {
		ostart ();
		otab ();
		oELSE ();
		odon ();
		oend ();

		eib->ElseIfStmt->ElseIfStmt.ElseIf = &If;

		sta_write_sta_bef (eib->ElseIfStmt);
		U_stack_push_selse (eib->ElseIfStmt);
	}

	oTab ();
	onod (eib->ElseIfStmt->ElseIfStmt.ElseIf);
	oLP ();
	onod (eib->ElseIfStmt->ElseIfStmt.Expr);
	oRP ();
	oTHEN ();
	oDon ();

	onod (eib->ElseIfBlock);
}

static write_elseif_endifs ()
{
	Node *elseif;

	while ((elseif = U_stack_pop_selse ()) != NN) {
		ostart ();
		otab ();
		oEND_IF ();
		odon ();
		oend ();

		sta_write_sta_aft (elseif);
	}
}

write_IfConstruct (node)
Node *node;
{
	struct S_IfConstruct *ifc = &node->IfConstruct;
	Node *elseif_block, *elseif_list, *temp;

	U_stack_clear_selse ();

	sta_write_sta_bef (ifc->IfThenStmt);

	ostart ();
	otab ();
	oIF ();
	oLP ();
	onod (ifc->IfThenStmt->IfThenStmt.Expr);
	oRP ();
	oTHEN ();
	odon ();
	oend ();

	write_pass (ifc->IfBlock);

	if ((elseif_list = ifc->ElseIfStmtBlockList) != NN)
		for (LIST_LOOP (elseif_block, elseif_list, temp))
			write_elseif_block (elseif_block);

	if (ifc->ElseStmt != NN) {
		ostart ();
		otab ();
		oELSE ();
		odon ();
		oend ();

		write_pass (ifc->ElseBlock);
	}

	write_elseif_endifs ();

	ostart ();
	otab ();
	oEND_IF ();
	odon ();
	oend ();

	sta_write_sta_aft (ifc->IfThenStmt);

	if (ifc->label_endif != NN)
		o_continue (ifc->label_endif);
}

write_IfStmt (node)
Node *node;
{
	struct S_IfStmt *ifs = &node->IfStmt;

	if (ifs->Stmt->com.typ != (Typ*) 0
	&& SHAPE_IS_NOT_NULL (ifs->Stmt->com.typ->shape)) {
		oIF ();
		oLP ();
		onod (ifs->Expr);
		oRP ();
		oTHEN ();
		odon ();
		oend ();

		ostart ();
		otab ();
		onod (ifs->Stmt);
		odon ();
		oend ();

		ostart ();
		otab ();
		oEND_IF ();
	}
	else
		ochl (node);
}
